/*
 * Copyright (c) 2006 Pepper Computer, Inc. All Rights Reserved.
 * PEPPER PROPRIETARY/CONFIDENTIAL.  Use is subject to license terms.
 */
package com.pepper.HW;

import java.awt.event.ActionEvent;

import org.jdom.Document;
import org.jdom.Element;

import com.pepper.platform.program.SectionList;
import com.pepper.platform.program.Section;
import com.pepper.platform.program.ActionEventWithParams;
import com.pepper.platform.program.actions.ProgramAbstractAction;
import com.pepper.platform.page.Page;
import com.pepper.platform.program.TabbedSectionMgr;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * <p>A framework Action class that saves the data items contained in the UI
 * widgets of a <code>WorldJavaSection</code> Section into the <code>WorldJavaSection</code>
 * object's current world XML instance file. More simply put: saves user-entered
 * world data from the world Java Section to disk.
 */
public class DoneJavaAction extends ProgramAbstractAction {

    /**
     * Handle for acessing the <code>HelloWorld</code> object, the application's
     * base class
     * */
    private HelloWorld helloWorld;

    /** Handle for accessing the worlds Section */
    protected Section worldsSection;

    /**
     * Constant representing the framework path for accessing the Worlds.xml XML
     * instance file
     * */
    public static final String WORLDS_XML = "data/Worlds";

    /**
     * <p>The Log instance, as required for every class that is to
     * write to the Keeper log file</p>
     */
    static Log log = LogFactory.getLog("com.placeholder.HW");


    public DoneJavaAction() {
    }

    /**
     * Saves the <code>WorldJavaSection</code> Section's current UI widget data
     * into the current world's XML instance file and displays the worlds sectionPage.
     *
     * @param event ActionEvent the framework event passed. Not used here.
     */
    public void actionPerformed(ActionEvent event) {

        //log presence in this method
        log.info("DoneJavaAction in process.");

        //get handle to HelloWorld
        this.helloWorld = (HelloWorld) getProgram();

        //save values in current WorldJavaSection widget's to underlying world XML file
        saveToXml();

        //display the worlds Section
        try {
            //get a handle to the WorldJavaSection
            worldsSection = helloWorld.getSection(WORLDS_XML);

            //switch focus to the worlds Section
            helloWorld.showSection(worldsSection);
        } catch (Exception e) {
            log.error("Could not display worlds Section from DoneJavaAction");
        }
    }

    /**
     * Helper method that saves the <code>WorldJavaSection</code> Section's
     * current UI widget data into the current world's XML instance file
     */
    private void saveToXml() {

        //get WorldJavaSection's current pageId, which shows the page ID of the underlying
        //world-specific XML instance file
        String pageId = helloWorld.worldJavaSection.getPageId();

        //get the data for the world XML instance page
        Page worldPage = getProgram().getPage(pageId, WORLDS_XML);

        //load the page from disk into memory if it is not yet loaded
        if (!worldPage.isLoaded()) {
            try {
                worldPage.load();
                //getGSP().writeStatus("XML instance file sucessfully loaded into memory");
            } catch (Exception e) {
                log.error("World's XML instance file failed to load into memory");
            }
        }

        //load worldPage XML data into JDOM Document
        Document worldData = worldPage.getPageData();

        synchronized (worldData) {
            //Load XML element data into Java widgets.
            //Note that the root element that is loaded is not the actual XMl file's
            //root but is rather the <page> element.
            Element root = worldData.getRootElement();

            //get the current value of each UI widget and save it in an appropriate
            //format into the corresponding element in the JDOM Document
            if (root != null) {
                Element worldNameElement = root.getChild("worldName");
                worldNameElement.setText(helloWorld.worldJavaSection.worldPanel.worldNameTextField.getText());

                Element worldRadiusElement = root.getChild("worldRadius");
                worldRadiusElement.setText(helloWorld.worldJavaSection.worldPanel.worldRadiusTextField.getText());

                Element yearLengthElement = root.getChild("yearLength");
                yearLengthElement.setText(helloWorld.worldJavaSection.worldPanel.yearLengthTextField.getText());

                Element dayLengthElement = root.getChild("dayLength");
                dayLengthElement.setText(helloWorld.worldJavaSection.worldPanel.dayLengthTextField.getText());

                Element distanceFromSunElement = root.getChild("distanceFromSun");
                distanceFromSunElement.setText(helloWorld.worldJavaSection.worldPanel.distanceFromSunTextField.getText());

                Element hasWaterElement = root.getChild("hasWater");
                if (helloWorld.worldJavaSection.worldPanel.hasWaterCheckBox.isSelected() == true) {
                    hasWaterElement.setText("true");
                } else {
                    hasWaterElement.setText("false");
                }

                Element hasLifeElement = root.getChild("hasLife");
                if (helloWorld.worldJavaSection.worldPanel.hasLifeCheckBox.isSelected() == true) {
                    hasLifeElement.setText("true");
                } else {
                    hasLifeElement.setText("false");
                }

                Element planVisitElement = root.getChild("planVisit");
                //getGSP().writeStatus("true");
                if (helloWorld.worldJavaSection.worldPanel.planVisitNow.isSelected()) {
                    planVisitElement.setText("1");
                } else if (helloWorld.worldJavaSection.worldPanel.planVisitSoon.isSelected()) {
                    planVisitElement.setText("2");
                } else if (helloWorld.worldJavaSection.worldPanel.planVisitNever.isSelected()) {
                    planVisitElement.setText("3");
                }

            } else {
                log.error("Root element not found in page ");
            }
        } //end synchronized block

        //Save the modifications made to the JDOM Document
        //to the world XML instance file.
        //Note: it is necessary to use the framework savePage() method in order to
        //to trigger the framework caching mechanism
        try {
            helloWorld.savePage(worldPage, false);
        } catch (Exception e) {
            log.error("Cannot save world XMl instance file to disk");
        }
    }
}
