/*
 * Copyright (c) 2006 Pepper Computer, Inc. All Rights Reserved.
 * PEPPER PROPRIETARY/CONFIDENTIAL.  Use is subject to license terms.
 */

package com.pepper.HW;

import java.awt.event.ActionEvent;

import org.jdom.Document;
import org.jdom.Element;

import com.pepper.platform.program.JavaSection;
import com.pepper.platform.program.Section;
import com.pepper.platform.program.ActionEventWithParams;
import com.pepper.platform.program.actions.ProgramAbstractAction;
import com.pepper.platform.page.Page;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * <p>A framework Action class that displays a <code>WorldJavaSection</code> Section.
 * The Action associates the Section with a particular world (to edit) by
 * receving an ID parameter that is a framework path to the world's XML instance
 * file. The Section's widgets are loaded with data items associated with the
 * specified world from the XML instance file. The user can edit the attributes
 * directly, although they are not saved into the XML instance file until the
 * DoneJava Action is triggered.
 */
public class EditJavaAction extends ProgramAbstractAction {

    /**
     * Handle for acessing the <code>HelloWorld</code> object, the application's
     * base class, required if writing to the status bar is done.
     */
    private HelloWorld helloWorld;

    /**Handle for accessing the <code>WorldJavaSection</code> */
    protected Section worldJavaSection;

    /** Constant for framework ID of Worlds.xml page instance file. */
    public static final String WORLDS_XML = "data/Worlds";

    /** Constant for framework ID of (pre-built) WorldJavaSection XML instance file. */
    public static final String WORLD_JAVA_SECTION = "data/WorldJavaSection";

    /**
     * <p>The Log instance, as required for every class that is to
     * write to the Keeper log file.</p>
    */
     static Log log = LogFactory.getLog("com.pepper.HW");

    /**
     * Catches pageId passed in as parameter
     */
     private String pageId;

    /** 
     * Empty constructor 
     */
    public EditJavaAction() {
    }

    /**
     * Displays the <code>WorldJavaSection</code> Section with its UI widgets
     * populated with data derived from the selected world's XML instance file.
     *
     * @param event ActionEvent the framework event that contains the <code>pageID</code>
     * parameter that specifies the framework path to the selected world's XML
     * instance file
     */
    public void actionPerformed(ActionEvent event) {

        log.info("EditJavaAction in process.");

        //get a handle to HelloWorld base class
        this.helloWorld = (HelloWorld) getProgram();

        if (event instanceof ActionEventWithParams) {
            //try to display the WorldJavaSection:
            try {
                //get a handle to the WorldJavaSection
                worldJavaSection = helloWorld.getSection(WORLD_JAVA_SECTION);

                //switch focus to the WorldJavaSection
                helloWorld.showSection(worldJavaSection);
            } catch (Exception e) {
                log.error("Could not display Java Section from EditJavaAction");
            }
            //get the pageId of the world page to display from the parameter passed to this Action
            this.pageId = ((ActionEventWithParams) event).getParam();

            //set JavaSection's pageId so it knows the id of the page it is to display
            helloWorld.worldJavaSection.setPageId(pageId);

            //Make a Page object for the world Page to be edited 
            //method's first param is the page ID to get, handed here from worlds sectionPage
            //method's second param is the id of the section (the worlds sectionPage) the page is in 
            Page worldPage = getProgram().getPage(pageId, WORLDS_XML);
            // load the page from disk into memory if it is not yet loaded
            if (!worldPage.isLoaded()) {
                try {
                    worldPage.load();
                } catch (Exception e){
                    log.error("World's XML instance file failed to load into memory");
                }
            }

            //create a Document object for the target world base page
            Document worldData = worldPage.getPageData();

            synchronized (worldData) {
                //Load XML element data into java widgets.
                //Note that the root element that is loaded is not the actual XML file's
                //root but is rather the <page> element.
                Element root = worldData.getRootElement();

                if (root != null) {
                    Element worldNameElement = root.getChild("worldName");
                    helloWorld.worldJavaSection.worldPanel.worldNameTextField.setText(worldNameElement.getText());

                    Element worldRadiusElement = root.getChild("worldRadius");
                    helloWorld.worldJavaSection.worldPanel.worldRadiusTextField.setText(worldRadiusElement.getText());

                    Element yearLengthElement = root.getChild("yearLength");
                    helloWorld.worldJavaSection.worldPanel.yearLengthTextField.setText(yearLengthElement.getText());

                    Element dayLengthElement = root.getChild("dayLength");
                    helloWorld.worldJavaSection.worldPanel.dayLengthTextField.setText(dayLengthElement.getText());

                    Element distanceFromSunElement = root.getChild("distanceFromSun");
                    helloWorld.worldJavaSection.worldPanel.distanceFromSunTextField.setText(distanceFromSunElement.getText());

                    Element hasWaterElement = root.getChild("hasWater");
                    if (hasWaterElement.getText().equals("true")) {
                        helloWorld.worldJavaSection.worldPanel.hasWaterCheckBox.setSelected(true);
                    } else {
                        helloWorld.worldJavaSection.worldPanel.hasWaterCheckBox.setSelected(false);
                    }

                    Element hasLifeElement = root.getChild("hasLife");
                    if (hasLifeElement.getText().equals("true")) {
                        helloWorld.worldJavaSection.worldPanel.hasLifeCheckBox.setSelected(true);
                    } else {
                        helloWorld.worldJavaSection.worldPanel.hasLifeCheckBox.setSelected(false);
                    }

                    Element planVisitElement = root.getChild("planVisit");
                    if (planVisitElement.getText().equals("1")) {
                        helloWorld.worldJavaSection.worldPanel.planVisitNow.setSelected(true);
                    } else if (planVisitElement.getText().equals("2")) {
                        helloWorld.worldJavaSection.worldPanel.planVisitSoon.setSelected(true);
                    } else if (planVisitElement.getText().equals("3")) {
                        helloWorld.worldJavaSection.worldPanel.planVisitNever.setSelected(true);
                    }
                } else {
                    log.error("Root element not found in page ");
                }
            }
        }
    }
}
