/*
 * Copyright 2006, Pepper Computer, Inc.  All rights reserved.
 * PEPPER PROPRIETARY/CONFIDENTIAL.  Use is subject to license terms.
 */
package com.pepper.HW;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.ComponentOrientation;

import javax.swing.border.EmptyBorder;
import javax.swing.BorderFactory;
import javax.swing.AbstractAction;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.BoxLayout;

import java.util.Properties;

import com.pepper.platform.program.AbstractPepperProgram;
import com.pepper.platform.program.JavaSectionComponent;
import com.pepper.guiutils.ToolBar;
import com.pepper.guiutils.ToolBarButton;
import com.pepper.platform.i18n.MessageCatalog;

 /**
 * <p>A Pepper Keeper framework Java Section that provides a user interface
 * for displaying and editing a world's data.</p>
 *    <p>All Pepper framework Java Sections must extend <code>JPanel</code>.</p>
 *    <p>Implementing the <code>JavaSectionComponent</code> interface hooks
 * <code>WorldJavaSection</code> into the framework, enabling such    things as
 * writing to the Keeper log and writing to the status bar.</P>
 *
 */
public class WorldJavaSection extends JPanel implements JavaSectionComponent {
    
    /** ToolBar for the Section */
    protected ToolBar toolbar = new ToolBar();
    
    /** JPanel for Section content. */
    protected JPanel panel_content = new JPanel();
    
    /** Handle for acessing <code>HelloWorld</code>, the application's base class. */
    public HelloWorld helloWorld;
    
    /**
     * The framework path to the particular world page's XML instance file being
     * displayed/edited.
     */
    private String pageId;
    
    /** A JPanel used for layout. */
    protected JPanel worldPanelMargin;
    
    /** A JPanel used for layout. */
    protected WorldPanel worldPanel;
    
    /**
     * Empty constructor.
     */
    public WorldJavaSection() {
    }
    
    /**
     * <p>Required method override for classes that implement the
     * <code>JavaSectionComponent</code> interface that is used here to create a
     * handle in <code>HelloWorld</code> to this class and to execute 
     * initiWorldJavaSection(), which initializes the Section's user interface. </p>
     * 
     * @param theProg is the AbstractPepperProgram bas class object tha tis passed 
     * automatically by the framework
     * @param params is a Properties object containing framework parameters
     * passed automatically by the framework
     */
    public void initComponent(AbstractPepperProgram theProg, Properties params) {
        //give HelloWorld (and therefore my Action classes) a handle to this class
        helloWorld = (HelloWorld) theProg;
        helloWorld.setJavaSection(this);
        
        //initialize the Section
        initWorldJavaSection();
    }
    
    /**
    * <p>Initialize the Section</p>
    */
    public void initWorldJavaSection() {
        
        //set layout and add toolbar and root JPanel
        this.setLayout(new BorderLayout());
        this.add(toolbar, BorderLayout.NORTH);
        this.add(panel_content, BorderLayout.CENTER);
        
        //create button assocaited with DoneJavaAction and set display text
        ToolBarButton b1 = new ToolBarButton(new DoneJavaAction(), "Done");
        
        //add buton to toolbar
        toolbar.addButton(b1);
        
        //WorldPanel class holds UI widgets.
        worldPanel = new WorldPanel();
        worldPanel.setLayout(new FlowLayout());
        worldPanel.setBorder(BorderFactory.createEtchedBorder());
        
        worldPanelMargin = new JPanel();
        worldPanelMargin.setLayout(new FlowLayout());
        worldPanelMargin.add(worldPanel);
        
        panel_content.add(worldPanelMargin, BorderLayout.CENTER);
    }
    
    /**
     * <p>Holds all display/input widgets that enable a user to edit a particular
     * world's data items.</p>
     */
    public class WorldPanel extends JPanel {
        
        /**
         * <p>Contains all widgets and provides a margin between their edges and the
         *  etched border of WorldPanel itself.</p>
         */
        protected JPanel marginPanel;
        
        /**
         * <p>GUI Label.</p>
         */
        protected JLabel worldName;
        
        /**
         * <p>GUI text input field.</p>
         */
        protected JTextField worldNameTextField;
        
        /**
         * <p>GUI Label.</p>
         */
        protected JLabel worldRadius;
        
        /**
         * <p>GUI text input field.</p>
         */
        protected JTextField worldRadiusTextField;
        
        /**
         * <p>GUI Label.</p>
         */
        protected JLabel yearLength;
        
        /**
         * <p>GUI text input field.</p>
         */
        protected JTextField yearLengthTextField;
        
        /**
         * <p>GUI Label.</p>
         */
        protected JLabel dayLength;
        
        /**
         * <p>GUI text input field.</p>
         */
        protected JTextField dayLengthTextField;
        
        /**
         * <p>GUI Label.</p>
         */
        protected JLabel distanceFromSun;
        
        /**
         * <p>GUI text input field.</p>
         */
        protected JTextField distanceFromSunTextField;
        
        /**
         * <p>GUI Label.</p>
         */
        protected JLabel hasWater;
        
        /**
         * <p>GUI checkbox input field.</p>
         */
        protected JCheckBox hasWaterCheckBox;
        
        /**
         * <p>GUI Label.</p>
         */
        protected JLabel hasLife;
        
        /**
         * <p>GUI checkbox input field.</p>
         */
        protected JCheckBox hasLifeCheckBox;
        
        /**
         * <p>JPanel to hold radio buttons</p>
         */
        protected JPanel planVisitPanel;
        
        /**
         * <p>GUI Label.</p>
         */
        protected JLabel planVisit;
        
        /**
         * <p>GUI radio button.</p>
         */
        protected JRadioButton planVisitNow;
        
        /**
         * <p>GUI radio button.</p>
         */
        protected JRadioButton planVisitSoon;
        
        /**
         * <p>GUI radio button.</p>
         */
        protected JRadioButton planVisitNever;
        
        /**
         * <p>Radio button group.</p>
         */
        protected ButtonGroup planVisitGroup;
        
        /**
         * <p>Contains all widgets used to display/edit each particular world's
         * data attributes.</p>
         */
        public WorldPanel() {
            //world name. This label derived from PackageStrings.properties
            //through the MessageCatalog, an approach that supports localization
            MessageCatalog catalog = helloWorld.getMessageCatalog();
            worldName = new JLabel(catalog.getString("JavaLabel.Name"), JLabel.RIGHT);
            worldNameTextField = new JTextField();
            
            //world radius
            worldRadius = new JLabel(catalog.getString("JavaLabel.Radius"), JLabel.RIGHT);
            worldRadiusTextField = new JTextField();
            
            //year length
            yearLength = new JLabel(catalog.getString("JavaLabel.YearLength"), JLabel.RIGHT);
            yearLengthTextField = new JTextField();
            
            //day length
            dayLength = new JLabel(catalog.getString("JavaLabel.DayLength"), JLabel.RIGHT);
            dayLengthTextField = new JTextField();
            
            //distance from sun
            distanceFromSun = new JLabel(catalog.getString("JavaLabel.DistanceFromSun"), JLabel.RIGHT);
            distanceFromSunTextField = new JTextField();
            
            //has water
            hasWater = new JLabel(catalog.getString("JavaLabel.HasWater"), JLabel.RIGHT);
            hasWaterCheckBox = new JCheckBox();
            
            //has life
            hasLife = new JLabel(catalog.getString("JavaLabel.HasLife"), JLabel.RIGHT);
            hasLifeCheckBox = new JCheckBox();
            
            //Plan visit: create label
            planVisit = new JLabel(catalog.getString("JavaLabel.PlanVisit"), JLabel.RIGHT);
            
            //plan visit: create radio buttons
            planVisitNow = new JRadioButton("Now");
            planVisitNow.setActionCommand("Now");
            planVisitNow.setSelected(false);
            planVisitSoon = new JRadioButton("Soon");
            planVisitSoon.setActionCommand("Soon");
            planVisitSoon.setSelected(false);
            planVisitNever = new JRadioButton("Never");
            planVisitNever.setActionCommand("Never");
            planVisitNever.setSelected(false);
            
            //plan visit: group the radio buttons
            planVisitGroup = new ButtonGroup();
            planVisitGroup.add(planVisitNow);
            planVisitGroup.add(planVisitSoon);
            planVisitGroup.add(planVisitNever);
            
            //plan visit: create panel to hold radio buttons and add buttons to it
            planVisitPanel = new JPanel();
            planVisitPanel.setLayout(new BoxLayout(planVisitPanel, BoxLayout.LINE_AXIS));
            planVisitPanel.add(planVisitNow);
            planVisitPanel.add(planVisitSoon);
            planVisitPanel.add(planVisitNever);
            
            //marginPanel wraps all widgets and provides a margin around them
            marginPanel = new JPanel();
            
            //worldPanel: add components
            marginPanel.setLayout(new GridLayout(0,2));
            marginPanel.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
            
            marginPanel.add(worldName);
            marginPanel.add(worldNameTextField);
            marginPanel.add(worldRadius);
            marginPanel.add(worldRadiusTextField);
            marginPanel.add(yearLength);
            marginPanel.add(yearLengthTextField);
            marginPanel.add(dayLength);
            marginPanel.add(dayLengthTextField);
            marginPanel.add(distanceFromSun);
            marginPanel.add(distanceFromSunTextField);
            marginPanel.add(hasWater);
            marginPanel.add(hasWaterCheckBox);
            marginPanel.add(hasLife);
            marginPanel.add(hasLifeCheckBox);
            marginPanel.add(planVisit);
            marginPanel.add(planVisitPanel);
            
            this.add(marginPanel);
        }
    }
    
    /**
     * <p>Informs the <code>WorldJavaSection</code> object of the framework page ID of the
     * XML instance page that the <code>WorldJavaSection</code> object is displaying.</p>
     *
     * @param pageId is the framework path to the XML instance file being
     * displayed/edited
     */
    protected void setPageId(String pageId) {
        this.pageId = pageId;
    }
    
    /**
     * <p>Gets the framework page ID of the XML instance page that the <code>WorldJavaSection</code>
     * object is displaying.</p>
     *
     * @param pageId is the framework path to the XML instance file being displayed/edited
     */
    protected String getPageId() {
        return pageId;
    }
}
